from unittest import TestCase
from neuropower import BUM
import numpy as np

class TestBUM(TestCase):
    def test_fpLL(self):
        np.random.seed(seed=100)
        testpeaks = np.vstack((np.random.uniform(0,1,10),np.random.uniform(0,0.2,10))).flatten()
        x = np.sum(BUM.fpLL([0.5,0.5],testpeaks))
        self.assertEqual(np.around(x,decimals=2),9.57)

    def test_fbumnLL(self):
        np.random.seed(seed=100)
        testpeaks = np.vstack((np.random.uniform(0,1,10),np.random.uniform(0,0.2,10))).flatten()
        x = BUM.fbumnLL([0.5,0.5],testpeaks)
        self.assertEqual(np.around(x,decimals=2)[0],-4.42)

    def test_bumOptim(self):
        np.random.seed(seed=100)
        testpeaks = np.vstack((np.random.uniform(0,1,10),np.random.uniform(0,0.2,10))).flatten()
        x = BUM.bumOptim(testpeaks,starts=1,seed=100)
        self.assertEqual(np.around(x['pi1'],decimals=2),0.29)



#!/usr/bin/env python

"""
Fit a beta-uniform mixture model to a list of p-values.
The BUM model is introduced in Pounds & Morris, 2003.
"""

import numpy as np
import scipy

def fpLL(pars,x):
	# Returns the gradient function of the BUM model
	a = pars[0]
	l = pars[1]
	dl = -sum((1-a*x**(a-1))/(a*(1-l)*x**(a-1)+l))
	da = -sum((a*(1-l)*x**(a-1)*np.log(x)+(1-l)*x**(a-1))/(a*(1-l)*x**(a-1)+l))
	return np.asarray([dl,da])

def fbumnLL(pars,x):
	#Returns the negative sum of the loglikelihood
	a = pars[0]
	l = pars[1]
	L = l+(1-l)*a*x**(a-1)
	negsumlog = -sum(np.log(L))
	return(negsumlog)

def EstimatePi1(x,starts=10,seed=None):
	# Returns the MLE estimator for pi1, with the shaping parameters and the value of the negative sum of the loglikelihood
	"""Searches the maximum likelihood estimator for the shape parameters of the BUM-model given a list of p-values"""
	seed = np.random.uniform(0,1000,1) if not 'seed' in locals() else seed
	a = np.random.uniform(0.05,0.95,(starts,))
	l = np.random.uniform(0.05,0.95,(starts,))
	best = []
	par = []
	x = np.asarray(x)
	x = [10**(-6) if y<= 10**(-6) else y for y in x] #optimiser is stuck when p-values == 0
	for i in range(0,starts):
		pars = np.array((a[i],l[i]))
		opt = scipy.optimize.minimize(fbumnLL,[pars[0],pars[1]],method='L-BFGS-B',args=(x,),jac=fpLL,bounds=((0.00001,1),(0.00001,1)))
		best.append(opt.fun)
		par.append(opt.x)
	minind=best.index(np.nanmin(best))
	bestpar=par[minind]
	pi1=1-(bestpar[1] + (1-bestpar[1])*bestpar[0])
	out={'maxloglikelihood': best[minind],
		'pi1': pi1,
		'a': bestpar[0],
		'lambda': bestpar[1]}
	return(out)



from unittest import TestCase
from neuropower import neuropowermodels
import numpy as np

class TestAltPDF(TestCase):
    def test_AltPDF_RFT(self):
        x = neuropowermodels.altPDF(peaks=[4.],mu=3.,sigma=1.,exc=2.,method="RFT")
        self.assertEqual(np.around(x,decimals=2)[0],0.29)

    def test_AltPDF_CS(self):
        x = neuropowermodels.altPDF(peaks=[4.],mu=3.,method="CS")
        self.assertEqual(np.around(x,decimals=2)[0],0.31)

class TestNulPDF(TestCase):
    def test_NulPDF_RFT(self):
        x = neuropowermodels.nulPDF(peaks=[4.],exc=2.,method="RFT")
        self.assertEqual(np.around(x,decimals=2)[0],0.04)

    def test_NulPDF_CS(self):
        x = neuropowermodels.nulPDF(peaks=[4.],method="CS")
        self.assertEqual(np.around(x,decimals=2)[0],0.01)

class TestAltCDF(TestCase):
    def test_AltCDF_RFT(self):
        x = neuropowermodels.altCDF(peaks=[4.],mu=3.,sigma=1.,exc=2.,method="RFT")
        self.assertEqual(np.around(x,decimals=2)[0],0.81)

    def test_AltCDF_CS(self):
        x = neuropowermodels.altCDF(peaks=[4.],mu=3.,method="CS")
        self.assertEqual(np.around(x,decimals=2)[0],0.16)

class TestNulCDF(TestCase):
    def test_NulCDF_RFT(self):
        x = neuropowermodels.nulCDF(peaks=[3.],exc=2.,method="RFT")
        self.assertEqual(np.around(x,decimals=2)[0],0.86)

    def test_NulCDF_CS(self):
        x = neuropowermodels.nulCDF(peaks=[3.],method="CS")
        self.assertEqual(np.around(x,decimals=2)[0],0.93)

class TestTruncTau(TestCase):
    def test_trunctau(self):
        x = neuropowermodels.TruncTau(4,1,3)
        self.assertEqual(np.around(x,decimals=2),0.21)

class TestMixPDF(TestCase):
    def test_MixPDF_RFT(self):
        x = neuropowermodels.mixPDF(peaks=[4.],pi1=0.5,mu=3.,sigma=1.,exc=2.,method="RFT")
        self.assertEqual(np.around(x,decimals=2),0.16)

    def test_MixPDF_CS(self):
        x = neuropowermodels.mixPDF(peaks=[4.],pi1=0.5,mu=3.,method="CS")
        self.assertEqual(np.around(x,decimals=2),0.16)

class TestMixPDF_SLL(TestCase):
    def test_MixPDF_SLL_RFT(self):
        np.random.seed(seed=100)
        testpeaks = np.random.uniform(2,10,30)
        x = neuropowermodels.mixPDF_SLL(pars=[4,1],peaks=testpeaks,pi1=0.5,exc=2.,method="RFT")
        self.assertEqual(np.around(x,decimals=2),156.03)

    def test_MixPDF_SLL_CS(self):
        np.random.seed(seed=100)
        testpeaks = np.random.uniform(-5,5,30)
        x = neuropowermodels.mixPDF_SLL(pars=[3],peaks=testpeaks,pi1=0.5,method="CS")
        self.assertEqual(np.around(x,decimals=2),451.62)

class TestModelfit(TestCase):
    def test_Modelfit_RFT(self):
        np.random.seed(seed=100)
        testpeaks = np.random.uniform(2,10,20)
        x = neuropowermodels.modelfit(peaks=testpeaks,pi1=0.5,exc=2.,starts=1,seed=20,method="RFT")
        self.assertEqual(np.around(x['mu'],decimals=2),6.10)

    def test_MixPDF_SLL_CS(self):
        np.random.seed(seed=100)
        testpeaks = np.random.uniform(-5,5,30)
        x = neuropowermodels.modelfit(peaks=testpeaks,pi1=0.5,starts=1,seed=20,method="CS")
        self.assertEqual(np.around(x['maxloglikelihood'],decimals=2),448.15)



import scipy.stats
import numpy as np
from neuropower import peakdistribution
import scipy.integrate as integrate

""" Fit a mixture model to a list of peak height T-values.
The model is introduced in the HBM poster:
http://www2.warwick.ac.uk/fac/sci/statistics/staff/academic-research/nichols/presentations/ohbm2015/Durnez-PeakPower-OHBM2015.pdf """

def altPDF(peaks,mu,sigma=None,exc=None,method="RFT"):
	"""
	altPDF: Returns probability density using a truncated normal
	distribution that we define as the distribution of local maxima in a
	GRF under the alternative hypothesis of activation
	parameters
	----------
	peaks: float or list of floats
		list of peak heigths
	mu:
	sigma:

	returns
	-------
	fa: float or list
		probability density of the peaks heights under Ha
	"""
	#Returns probability density of the alternative peak distribution
	peaks = np.asarray(peaks)
	if method == "RFT":
		# assert type(sigma) is in [float, int]
		# assert sigma is not None
		ksi = (peaks-mu)/sigma
		alpha = (exc-mu)/sigma
		num = 1/sigma * scipy.stats.norm.pdf(ksi)
		den = 1. - scipy.stats.norm.cdf(alpha)
		fa = num/den
	elif method == "CS":
		fa = [peakdistribution.peakdens3D(y-mu,1) for y in peaks]
	return fa

def nulPDF(peaks,exc=None,method="RFT"):
	#Returns probability density of the null peak distribution
	peaks = np.asarray(peaks)
	if method == "RFT":
		f0 = exc*np.exp(-exc*(peaks-exc))
	elif method == "CS":
		f0 = [peakdistribution.peakdens3D(x,1) for x in peaks]
	return f0

def altCDF(peaks,mu,sigma=None,exc=None,method="RFT"):
	# Returns the CDF of the alternative peak distribution
	peaks = np.asarray(peaks)
	if method == "RFT":
		ksi = (peaks-mu)/sigma
		alpha = (exc-mu)/sigma
		Fa = (scipy.stats.norm.cdf(ksi) - scipy.stats.norm.cdf(alpha))/(1-scipy.stats.norm.cdf(alpha))
	elif method == "CS":
		Fa = [integrate.quad(lambda x:peakdistribution.peakdens3D(x,1),-20,y)[0] for y in peaks-mu]
	return Fa

def TruncTau(mu,sigma,exc):
	num = scipy.stats.norm.cdf((exc-mu)/sigma)
	den = 1-scipy.stats.norm.pdf((exc-mu)/sigma)
	tau = num/den
	return tau

def nulCDF(peaks,exc=None,method="RFT"):
	# Returns the CDF of the null peak distribution
	peaks = np.asarray(peaks)
	if method == "RFT":
		F0 = 1-np.exp(-exc*(peaks-exc))
	elif method == "CS":
		F0 = [integrate.quad(lambda x:peakdistribution.peakdens3D(x,1),-20,y)[0] for y in peaks]
	return F0

def mixPDF(peaks,pi1,mu,sigma=None,exc=None,method="RFT"):
	# returns the PDF of the mixture of null and alternative distribution
	peaks = np.array(peaks)
	if method == "RFT":
		f0=nulPDF(peaks,exc=exc,method="RFT")
		fa=altPDF(peaks,mu,sigma=sigma,exc=exc,method="RFT")
	elif method == "CS":
		f0 = nulPDF(peaks,method="CS")
		fa = altPDF(peaks,mu,method="CS")
	f=[(1-pi1)*x + pi1*y for x, y in zip(f0, fa)]
	return(f)

def mixPDF_SLL(pars,peaks,pi1,exc=None,method="RFT"):
	# Returns the negative sum of the loglikelihood of the PDF with RFT
	mu = pars[0]
	if method == "RFT":
		sigma = pars[1]
		f = mixPDF(peaks,pi1=pi1,mu=mu,sigma=sigma,exc=exc,method="RFT")
	elif method == "CS":
		f = mixPDF(peaks,pi1=pi1,mu=mu,method="CS")
	LL = -sum(np.log(f))
	return(LL)

def modelfit(peaks,pi1,exc=None,starts=1,seed=None,method="RFT"):
	# Searches the maximum likelihood estimator for the mixture distribution of null and alternative
	peaks = np.asarray(peaks)
	seed = np.random.uniform(0,1000,1) if not 'seed' in locals() else seed
	mus = np.random.uniform(exc+(1./exc),10,(starts,)) if method == "RFT" else np.random.uniform(0,10,(starts,))
	seed = np.random.uniform(0,1000,1) if not 'seed' in locals() else seed
	sigmas = np.random.uniform(0.1,10,(starts,)) if method=="RFT" else np.repeat(None,starts)
	best = []
	par = []
	for i in range(0,starts):
		if method == "RFT":
			opt = scipy.optimize.minimize(mixPDF_SLL,[mus[i],sigmas[i]],method='L-BFGS-B',args=(peaks,pi1,exc,method),bounds=((exc+(1./exc),50),(0.1,50)))
		elif method == "CS":
			opt = scipy.optimize.minimize(mixPDF_SLL,[mus[i]],method='L-BFGS-B',args=(peaks,pi1,exc,method),bounds=((0,50),))
		best.append(opt.fun)
		par.append(opt.x)
	minind=best.index(np.nanmin(best))
	out={'maxloglikelihood': best[minind],
			'mu': par[minind][0],
			'sigma': par[minind][1] if method == "RFT" else 'nan'}
	return out

def threshold(peaks,pvalues,FWHM,voxsize,nvox,alpha=0.05,exc=None,method="RFT"):
	# only RFT
	peakrange = np.arange(exc,15,0.001)
	pN = 1-nulCDF(np.array(peakrange),exc=exc)
	# smoothness
	FWHM_vox = np.asarray(FWHM)/np.asarray(voxsize)
	resels = nvox/np.product(FWHM_vox)
	pN_RFT = resels*np.exp(-peakrange**2/2)*peakrange**2
	cutoff_UN = np.min(peakrange[pN<alpha])
	cutoff_BF = np.min(peakrange[pN<alpha/len(peaks)])
	cutoff_RFT = np.min(peakrange[pN_RFT<alpha])
	#Benjamini-Hochberg
	pvals_sortind = np.argsort(pvalues)
	pvals_order = pvals_sortind.argsort()
	FDRqval = pvals_order/float(len(pvalues))*0.05
	reject = pvalues<FDRqval
	if reject.any():
		FDRc = np.max(pvalues[reject])
	else:
		FDRc = 0
	cutoff_BH = 'nan' if FDRc==0 else min(peakrange[pN<FDRc])
	out = {'UN':cutoff_UN,'BF':cutoff_BF,'RFT':cutoff_RFT,'BH':cutoff_BH}
	return out

def BH(pvals,alpha):
	pvals_sortind = np.argsort(pvals)
	pvals_order = pvals_sortind.argsort()
	FDRqval = pvals_order/float(len(pvals))*alpha
	reject = pvals<FDRqval
	if np.sum(reject)==0:
		FDRc=0
	else:
		FDRc = np.max(pvals[reject])
	return FDRc



import numpy as np
import pandas as pd

""" Extract local maxima from a spm, return a csv file with variables:
- x coordinate
- y coordinate
- z coordinate
- peak height """

def PeakTable(spm,exc,mask):
	# make a new array with an extra row/column/plane around the original array
	spm_newdim = tuple(map(lambda x: x+2,spm.shape))
	spm_ext = np.zeros((spm_newdim))
	msk_ext = np.zeros((spm_newdim))
	spm_ext.fill(-100)
	spm_ext[1:(spm.shape[0]+1),1:(spm.shape[1]+1),1:(spm.shape[2]+1)] = spm
	msk_ext[1:(spm.shape[0]+1),1:(spm.shape[1]+1),1:(spm.shape[2]+1)] = mask
	spm_ext = spm_ext * msk_ext
	shape = spm.shape
	spm = None
	# open peak csv
	labels = ['x','y','z','peak']
	peaks = pd.DataFrame(columns=labels)
	# check for each voxel whether it's a peak, if it is, add to table
	for m in range(1,shape[0]+1):
		for n in range(1,shape[1]+1):
			for o in range(1,shape[2]+1):
				surroundings = None
				res = None
				if spm_ext[m,n,o]>exc:
					surroundings=[spm_ext[m-1,n-1,o-1],
					spm_ext[m-1,n-1,o],
					spm_ext[m-1,n-1,o+1],
					spm_ext[m-1,n,o-1],
					spm_ext[m-1,n,o],
					spm_ext[m-1,n,o+1],
					spm_ext[m-1,n+1,o-1],
					spm_ext[m-1,n+1,o],
					spm_ext[m-1,n+1,o+1],
					spm_ext[m,n-1,o-1],
					spm_ext[m,n-1,o],
					spm_ext[m,n-1,o+1],
					spm_ext[m,n,o-1],
					spm_ext[m,n,o+1],
					spm_ext[m,n+1,o-1],
					spm_ext[m,n+1,o],
					spm_ext[m,n+1,o+1],
					spm_ext[m+1,n-1,o-1],
					spm_ext[m+1,n-1,o],
					spm_ext[m+1,n-1,o+1],
					spm_ext[m+1,n,o-1],
					spm_ext[m+1,n,o],
					spm_ext[m+1,n,o+1],
					spm_ext[m+1,n+1,o-1],
					spm_ext[m+1,n+1,o],
					spm_ext[m+1,n+1,o+1]]
					if spm_ext[m,n,o] > np.max(surroundings):
						res =pd.DataFrame(data=[[m-1,n-1,o-1,spm_ext[m,n,o]]],columns=labels)
						peaks=peaks.append(res)
	peaks = peaks.set_index([range(len(peaks))])
	return peaks



from unittest import TestCase
from neuropower import cluster
import numpy as np

class TestCluster(TestCase):
    def test_cluster_output(self):
        np.random.seed(seed=100)
        testSPM = np.random.rand(4,4,4)
        mask = np.zeros((4,4,4))+1
        tab = cluster.cluster(testSPM,0.5,mask)
        self.assertEqual(np.around(tab.peak[1],decimals=2),0.98)
        self.assertEqual(len(tab),6)



from unittest import TestCase
from neuropower import peakdistribution
import numpy as np

class TestPeakDistribution(TestCase):
    def test_peakdistribution(self):
        x = peakdistribution.peakdens3D(2,1)
        self.assertEqual(np.around(x,decimals=2),0.48)



from setuptools import setup, find_packages

def readme():
    with open('README.rst') as f:
        return f.read()

setup(
    # Application name:
    name='neuropower',

    # Version number:
    version='0.2.7',

    # Author details
    author='Joke Durnez',
    author_email = 'joke.durnez@gmail.com',
    maintainer = 'Joke Durnez',
    maintainer_email = 'joke.durnez@gmail.com',

    # Packages
    packages=find_packages(),

    #Details
    url = 'https://github.com/neuropower/neuropower-core',
    description='A package to perform power analyses for neuroimaging data',
    long_description=readme(),
    license='MIT',
    keywords='statistics power fmri neuroimaging inference samplesize',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7'
    ],
    test_suite='nose.collector',
    install_requires=[
        'numpy==1.11.0',
        'scipy==0.17.0',
        'nibabel',
        'pandas',
        'nose==1.3.7'
        ]
    )
